unit setTool;

(*
     unit name        : setTool.pas
     release          : 0.1
     author           : Sven BOBO Bobrowski
     description      : wrapps functions for
                         + (de)registring for OCX and TBL files
                         + associates a program with a file
                         + adding/removing menu extensions to the default
                           file popup menu in the explorer
                         + adding mime type to a file
                         + adding clsid identifier to a file
                         + creating links to files to the start menu
                         + copying and decompressing z files (see zTools!)
     requirements     : delphi 3
                         + unit setData.pas and setData.dfm
     platforms        : developed under nt 4.0 (sp 3), should worked under 95
     copyright        : the code is copyright (c) 1997 Sven BOBO Bobrowski
     freeware         : use the code if you needed, no limiteds are maked by me
*)

interface

uses
    Windows, Classes, SysUtils, ComObj, ComCtrls, StdCtrls, FileCtrl, Forms,
    setData, ZLib, ZArcTyp;

const
  BufferSize = 4096;
  CMPError_None = 0;

type
  TRegisterMode = (regRegister, regUnregister);
  TDecompParameter = record
    strTmpDir : String;            // the temporary dir
    strArchivName : String;        // the name of the z archiv
    strSourcePath : String;        // the path to the archiv
    strDestPath : String;          // the path to the destination directory
    objMsgFile : TObject;          // TLabel object for file message
    objProgress : TObject;         // TProgressBar object for progress
    objStatus : TNotifyEvent;      // Callback procedure
    objLog : TObject;              // optional log list
    optRename : Boolean;           // uses a other name for the temp file
                                   // if source and dest in the same directory 
  end;

function GetEnvStr(strEnvVar : String) : String;
(* Comment: Gathers a envirenoment variable from the windows system
   Parametes:
     strEnvVar: The identifier of the variable
   Return value:
     a string with the value, else ''
   Sample:
     TempDir := GetEnvStr('TEMP')
*)

function RegisterOLEFile(const StrFileName : String;Mode : TRegisterMode) : Boolean;
(* Comment: Registers or UnRegisters any type library (*.tlb),
            any OCX (*.ocx) or any OLE server exe.
   Parametes:
     strFileName: The file name of the executable to be registered.
     mode:        regRegister if the ole intrfaces in the file should
                  be registerd
                  regUregister if the ole intrfaces in the file should
                  be unregisterd
   Return value:
     TRUE if the registration suceeded, else FALSE.
   Sample:
     Success := RegisterOLEFile('%sytem%\sample.ocx', regRegister)
       or
     Success := RegisterOLEFile('%sytem%\sample.ocx', regUnregister)
*)

procedure RegisterAssociation(StrExeFuelPath, StrExt, StrType : String);
(* Comment: Registers any file with a program
   Parametes:
     strExeFuelPath: The fuel path to the associated program
     strExt: The file extension of the associated file
     strType: The description of the file type
   Example:
     RegisterAssociation('c:\program files\example.exe', '.smp', 'samplefile');
*)

procedure RegisterAssociationCommand(StrExeFuelPath, StrMenuEntry, StrExt, StrType : String);
(* Comment: Registers any file with a program
   Parametes:
     strExeFuelPath: The fuel path to the associated program
     StrMenuEntry: The name of the entry in the popup menu
     strExt: The file extension of the associated file
     strType: The description of the file type
   Example:
     RegisterAssociationCommand('c:\program files\example.exe', 'View', '.smp', 'smp file');
*)

procedure RegisterAssociationContentType(StrExeFuelPath, StrContent, StrExt, StrType : String);
(* Comment: Registers any file with a program
   Parametes:
     strExeFuelPath: The fuel path to the associated program
     strContent: The content string
     strExt: The file extension of the associated file
     strType: The description of the file type
   Example:
     RegisterAssociationContentType('c:\program files\example.exe', 'image/gif', '.smp', 'smp file');
*)

procedure RegisterAssociationCLSID(StrCLSID, StrExt, StrType : String);
(* Comment: Registers any file with a ActiveX CLSID
   Parametes:
     strCLSID: The CLSID string (example: {25336920-03F9-11cf-8FD0-00AA00686F13})
     strExt: The file extension of the associated file
     strType: The description of the file type
   Example:
     RegisterAssociationCLSID('{25336920-03F9-11cf-8FD0-00AA00686F13}', '.smp', 'smp file');
*)

procedure RegisterAssociationDefaultIcon(strDefaultIcon, StrExt, StrType : String);
(* Comment: Registers any file with a symbol
   Parametes:
     strDefaultIcon: The string of the associated icon ('[Program Path (short!)],[Icon ID])
     strExt: The file extension of the associated file
     strType: The description of the file type
   Example:
     RegisterAssociationDefaultIcon('F:\PROGRA~1\EXAMPLE\EXAMPLE.exe,0', '.smp', 'smp file');
*)

procedure UnRegisterAssociation(StrExt, StrType : String);
(* Comment: Deletes the entries for a file extension in the registry
   Parameters:
     strExt: The file extension of the associated file
     strType: The description of the file type
*)

procedure CreateProgramLink(strExeFuelPath, strIcon : String);
(* Comment: Creates a entry in the startup menu
   Parametes:
     strExeFuelPath: The fuel path to the file
     strIcon: The name of the menu entry
   Example:
     CreateProgramLink('example.exe', 'Example 1.00');
*)

function CopyAndDecompressZArchiv(dcmpParam : TDecompParameter) : Integer;
(* Comment: Decompress a z Tools archiv
   Parametes:
     dcmpParam: record block with required informations
   Return:
     0 = ok, else a value > 0
*)

implementation

uses
  Registry, ActiveX, ShellAPI;

const
  EOL = #13#10;

(* GetEnvStr *)
function GetEnvStr(strEnvVar : String) : String;
var
   PVar : Array[0..254] of Char;
   Buf  : PChar;
begin
     GetMem(Buf, 255);
     StrPCopy(PVar, strEnvVar);
     GetEnvironmentVariable(PVar, Buf, 255);
     Result := StrPas(Buf);
     FreeMem(Buf, 255);
end;

(* OLERegisterDLLFile *)
function OLERegisterDLLFile(StrFileName : String;Mode : TRegisterMode) : Boolean;
type
    TOleRegister = function : HResult;
var
   hLib  : THandle;
   fnAdr : TFarProc;
begin
     Result := False;
     hLib := LoadLibrary(PChar(StrFileName));
     if (hLib > 0) then
     begin
          try
             if (Mode = regRegister) then
                fnAdr := GetProcAddress(hLib, PChar('DllRegisterServer'))
             else
                 fnAdr := GetProcAddress(hLib, PChar('DllUnregisterServer'));
             if (fnAdr <> nil) then Result := (TOleRegister(fnAdr) >= 0);
          finally
                 FreeLibrary(hLib);
          end;
     end;
end;

(* OLERegisterTLBFile *)
function OLERegisterTLBFile(StrFileName : String;Mode : TRegisterMode) : Boolean;
type
  TUnRegTlbProc = function (const libID: TGUID; wVerMajor, wVerMinor: Word;
    lcid: TLCID; syskind: TSysKind): HResult; stdcall;
var
   wstrFileName   : WideString;
   wstrDocName    : WideString;
   iTypeLibrary   : ITypeLib;
   pLibAttr       : PTLibAttr;
   hOleAutLib     : THandle;
   fnUnRegTlbProc : TUnRegTlbProc;
begin
     Result := True;
     try
        wstrFileName := strFileName;
        OLECheck(LoadTypeLib(PWideChar(wstrFileName), iTypeLibrary));
        OLECheck(iTypeLibrary.GetLibAttr(pLibAttr));
        try
           if (Mode = regRegister) then
           begin
                OleCheck(iTypeLibrary.GetDocumentation(-1, nil, nil, nil, @wstrDocName));
                wstrDocName := ExtractFilePath(wstrDocName);
                OleCheck(RegisterTypeLib(iTypeLibrary, PWideChar(wstrFileName ), PWideChar(wstrDocName)));
           end else
           begin
                hOleAutLib := GetModuleHandle('OLEAUT32.DLL');
                if (hOleAutLib <> 0) then
                begin
                     @fnUnRegTlbProc := GetProcAddress(hOleAutLib, 'UnRegisterTypeLib');
                     OleCheck(fnUnRegTlbProc(
                       pLibAttr^.Guid,
                       pLibAttr^.wMajorVerNum,
                       pLibAttr^.wMinorVerNum,
                       pLibAttr^.LCID,
                       pLibAttr^.SysKind
                     ));
                end;
           end;
        finally
               iTypeLibrary.ReleaseTLibAttr(pLibAttr);
        end;
     except
           Result := False;
     end;
end;

(* OLERegisterEXEFile *)
function OLERegisterEXEFile(StrFileName : String;Mode : TRegisterMode) : Boolean;
begin
     if (Mode = regRegister) then
        Result := (ShellExecute(0, 'open', PChar(StrFileName), '/RegServer', Nil, SW_SHOWNORMAL) > 32)
     else
         Result := (ShellExecute(0, 'open', PChar(strFileName), '/UnregServer', Nil, SW_SHOWNORMAL) > 32);
end;

(* RegisterOLEFile *)
function RegisterOLEFile(const StrFileName : String;Mode : TRegisterMode) : Boolean;
var
   strExt : String;
begin
     Result := FALSE;
     if (FileExists(StrFileName)) then
     begin
          strExt := ExtractFileExt(StrFileName);
          if (strExt <> '') then
          begin
               strExt := Copy(strExt, 2, Length(strExt));
               if
                 (CompareText (strExt, 'OCX') = 0) or
                 (CompareText (strExt, 'DLL') = 0) then
                              Result := OLERegisterDLLFile (strFileName, mode)
               else
               if (CompareText (strExt, 'TLB') = 0) then
                  Result := OLERegisterTLBFile (strFileName, mode)
               else
               if (CompareText (strExt, 'EXE') = 0) then
                  Result := OLERegisterEXEFile (strFileName, mode)
          end;
     end;
end;

(* RegisterAssociation *)
procedure RegisterAssociation(StrExeFuelPath, StrExt, StrType : String);
var
   reg : TRegistry;
begin
     reg := TRegistry.Create;
     try
        with reg do
        begin
             RootKey := HKEY_CLASSES_ROOT;
             if (OpenKey (strExt, TRUE)) then WriteString ('', strType);
             CloseKey;
             RootKey := HKEY_CLASSES_ROOT;
             if
               (OpenKey (strType, TRUE)) and
               (OpenKey ('shell', TRUE)) and
               (OpenKey ('open',TRUE)) and
               (OpenKey ('command', TRUE)) then
                        WriteString ('', '"' + StrExeFuelPath + '" "%1"');
             CloseKey;
        end;
     finally
            reg.Free;
     end;
end;

(* RegisterAssociationCommand *)
procedure RegisterAssociationCommand(StrExeFuelPath, StrMenuEntry, StrExt, StrType : String);
var
   reg : TRegistry;
begin
     reg := TRegistry.Create;
     try
        with reg do
        begin
             RootKey := HKEY_CLASSES_ROOT;
             if (OpenKey (strExt, TRUE)) then WriteString ('', strType);
             CloseKey;
             RootKey := HKEY_CLASSES_ROOT;
             if
               (OpenKey (strType, TRUE)) and
               (OpenKey ('shell', TRUE)) and
               (OpenKey (StrMenuEntry,TRUE)) and
               (OpenKey ('command', TRUE)) then
                        WriteString ('', '"' + StrExeFuelPath + '" "%1"');
             CloseKey;
        end;
     finally
            reg.Free;
     end;
end;

(* RegisterAssociationContentType *)
procedure RegisterAssociationContentType(StrExeFuelPath, StrContent, StrExt, StrType : String);
var
   reg : TRegistry;
begin
     reg := TRegistry.Create;
     try
        with reg do
        begin
             RootKey := HKEY_CLASSES_ROOT;
             if (OpenKey (strExt, TRUE)) then
             begin
                  WriteString('', strType);
                  WriteString('Content Type', StrContent);
             end;
             CloseKey;
             RootKey := HKEY_CLASSES_ROOT;
             if
               (OpenKey (strType, TRUE)) and
               (OpenKey ('shell', TRUE)) and
               (OpenKey ('open',TRUE)) and
               (OpenKey ('command', TRUE)) then
                        WriteString ('', '"' + StrExeFuelPath + '" "%1"');
             CloseKey;
        end;
     finally
            reg.Free;
     end;
end;

(* RegisterAssociationCLSID *)
procedure RegisterAssociationCLSID(StrCLSID, StrExt, StrType : String);
var
   reg : TRegistry;
begin
     reg := TRegistry.Create;
     try
        with reg do
        begin
             RootKey := HKEY_CLASSES_ROOT;
             if (OpenKey (strExt, TRUE)) then WriteString ('', strType);
             CloseKey;
             RootKey := HKEY_CLASSES_ROOT;
             if
               (OpenKey (strType, TRUE)) and
               (OpenKey ('CLSID', TRUE)) then
                        WriteString ('', '"' + StrCLSID+ '"');
             CloseKey;
        end;
     finally
            reg.Free;
     end;
end;

(* RegisterAssociationDefaultIcon *)
procedure RegisterAssociationDefaultIcon(strDefaultIcon, StrExt, StrType : String);
var
   reg : TRegistry;
begin
     reg := TRegistry.Create;
     try
        with reg do
        begin
             RootKey := HKEY_CLASSES_ROOT;
             if (OpenKey (strExt, TRUE)) then WriteString ('', strType);
             CloseKey;
             RootKey := HKEY_CLASSES_ROOT;
             if
               (OpenKey (strType, TRUE)) and
               (OpenKey ('DefaultIcon', TRUE)) then
                        WriteString ('', '"' + strDefaultIcon + '"');
             CloseKey;
        end;
     finally
            reg.Free;
     end;
end;

(* UnRegisterAssociation *)
procedure UnRegisterAssociation(StrExt, StrType : String);
var
   reg : TRegistry;
begin
     reg := TRegistry.Create;
     try
        with reg do
        begin
             RootKey := HKEY_CLASSES_ROOT;
             if
               (OpenKey (strType, FALSE)) and
               (OpenKey ('shell', FALSE)) then
                        DeleteKey ('Register');
             CloseKey;
             RootKey := HKEY_CLASSES_ROOT;
             if
               (OpenKey (strType, FALSE)) and
               (OpenKey ('shell', FALSE)) then
                        DeleteKey ('Unregister');
             CloseKey;
        end;
     finally
            reg.Free;
     end;
end;

(* CreateProgramLink *)
procedure CreateProgramLink(strExeFuelPath, strIcon : String);
var
   prMacro   : String;
   prName    : String;
   prIcon    : String;
   prCmd     : Array[0..255] of Char;
begin
     ToolsData.DDEClient.OpenLink;

     prName := '"' + strExeFuelPath + '"';
     prIcon := strIcon;
     prMacro:=Format('[AddItem(%s)]', [prName+chr(44)+prIcon+chr(44)+',,']) + #13#10;
     StrPCopy(prCmd,prMacro);
     ToolsData.DDEClient.ExecuteMacro(prCmd, True);

     ToolsData.DDEClient.CloseLink;
end;

function CopyAndDecompressZArchiv(dcmpParam : TDecompParameter) : Integer;
var
   Quelle     : TFileStream;
   zHeader    : TZHeader;
   zTable     : TZTable;
   zTableData : TZTableData;
   Buffer     : Array[0 .. BufferSize - 1] of Byte;
   InFile     : TStream;
   OutFile    : TStream;
   ZStream    : TCustomZLibStream;
   Dest       : TFileStream;
   i          : Integer;
   OldPos     : Longint;
   Bloecke,
   Count      : Integer;
   Rest       : Integer;
   Index      : Integer;
   Proz       : Integer;

   function ChangeFileName(const Value : String) : String;
   var
      Tmp : String;
   begin
        Tmp := Value;
        Delete(Tmp, Length(Tmp), 1);
        Tmp := Tmp + '$';
        Result := Tmp;
   end;

begin
     Result := CMPError_None;

     // archiv lesen
     quelle := tfilestream.create(
       dcmpParam.strSourcePath + dcmpParam.strArchivName,
       fmOpenRead
     );

     // analysieren
     quelle.readbuffer(zheader, sizeof(zheader));
     quelle.seek(zheader.tableoffset, soFromBeginning);
     quelle.readbuffer(ztable, sizeof(ztable));

     // dateien entpacken
     for i := 1 to ztable.count do
     begin
          quelle.readbuffer(ZTableData, sizeof(ZTableData));
          OldPos := quelle.position;

          TLabel(dcmpParam.objMsgFile).Caption := format('Kopiere: %s', [ZTableData.FileName]);
          Application.ProcessMessages;

          TProgressBar(dcmpParam.objProgress).Max := 100;
          Application.ProcessMessages;

          // temp file erzeugen
          if dcmpParam.optRename then
             Dest := TFileStream.Create(ChangeFileName(dcmpParam.strTmpDir + ZTableData.FileName), fmCreate)
          else
              Dest := TFileStream.Create(dcmpParam.strTmpDir + ZTableData.FileName, fmCreate);
          quelle.Seek(ZTableData.OffSet, soFromBeginning);

          Bloecke := Trunc(ZTableData.Size / BufferSize);
          Rest    := ZTableData.Size - (Bloecke * BufferSize);
          Index := 1;

          repeat
                if (Index <= Bloecke) then
                begin
                     Count := quelle.Read(Buffer, BufferSize);
                     Dest.Write(Buffer, Count);
                end else
                begin
                     Count := quelle.Read(Buffer, Rest);
                     Dest.Write(Buffer, Count);
                end;
                inc(Index, 1);
                Proz := Round(( (Quelle.Position / Quelle.Size) * 100));
                TProgressBar(dcmpParam.objProgress).Position := Proz;
                Application.ProcessMessages;
          until Index > (Bloecke + 1);
          Dest.Free;

          TLabel(dcmpParam.objMsgFile).Caption := format('Entpacke: %s', [ZTableData.FileName]);
          Application.ProcessMessages;

          // dekomprimieren
          if dcmpParam.optRename then
             InFile := TFileStream.Create(ChangeFileName(dcmpParam.strTmpDir + ZTableData.FileName), fmOpenRead)
          else
              InFile := TFileStream.Create(dcmpParam.strTmpDir + ZTableData.FileName, fmOpenRead);

          TProgressBar(dcmpParam.objProgress).Max := InFile.Size;
          Application.ProcessMessages;

          try
             OutFile := TFileStream.Create(dcmpParam.strDestPath + ZTableData.FileName, fmCreate);
             try
                ZStream := TDecompressionStream.Create(InFile);
                try
                   TDecompressionStream(ZStream).OnProgress := dcmpParam.objStatus;
                   while True do
                   begin
                        Count := ZStream.Read(Buffer, BufferSize);
                        if Count <> 0 then OutFile.WriteBuffer(Buffer, Count) else Break;
                   end;
                finally
                       ZStream.Free;
                end;
             finally
                    OutFile.Free;
             end;
          finally
                 InFile.Free;
          end;

          // optional loggen
          if dcmpParam.objLog <> Nil then
             with TStringList(dcmpParam.objLog) do
                  Insert(Count, 'FILE,' + dcmpParam.strDestPath + ZTableData.FileName);

          // tempfile lschen
          if dcmpParam.optRename then
             DeleteFile(ChangeFileName(dcmpParam.strTmpDir + ZTableData.FileName))
          else
              DeleteFile(dcmpParam.strTmpDir + ZTableData.FileName);
              
          quelle.Seek(OldPos, soFromBeginning);
     end;
     quelle.destroy;
end;

end.
